/**
 * Reading Progress. Exclusively on Envato Market: https://1.envato.market/42themeCC
 * @encoding     UTF-8
 * @version      2.0.0
 * @copyright    Copyright (C) 2016 - 2021 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

/* scrollprogress v3.0.2 - 2017 Jeremias Menichelli - MIT License */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.ScrollProgress = factory());
}(this, (function () { 'use strict';

  /**
   * Fallback noop function
   * @method noop
   * @returns {undefined}
   */
  function noop() {}

  /**
   * ScrollProgress class constructor
   * @constructor ScrollProgress
   * @param {Function} handleUpdate method to call on scroll update
   * @returns {undefined}
   */
  var ScrollProgress = function(handleUpdate) {
    // assign function to call on update
    this._handleUpdate = typeof handleUpdate === 'function'
      ? handleUpdate
      : noop;

    // set initial values
    this._viewportHeight = this._getViewportHeight();
    this._viewportWidth = this._getViewportWidth();

    this._progress = this._getProgress();

    // trigger initial update function
    this._handleUpdate(this._progress.x, this._progress.y);

    // bind event functions
    this._onScroll = this._onScroll.bind(this);
    this._onResize = this._onResize.bind(this);

    // add event listeners
    window.addEventListener('scroll', this._onScroll);
    window.addEventListener('resize', this._onResize);
  };

  /**
   * Get vertical trajectory of the viewport
   * @method _getViewportHeight
   * @returns {Number}
   */
  ScrollProgress.prototype._getViewportHeight = function() {
    return document.body.scrollHeight - window.innerHeight;
  };

  /**
   * Get horizontal trajectory of the viewport
   * @method _getViewportWidth
   * @returns {Number}
   */
  ScrollProgress.prototype._getViewportWidth = function() {
    return document.body.scrollWidth - window.innerWidth;
  };

  /**
   * Get scroll progress on both axis
   * @method _getProgress
   *
   * @returns {Object}
   **/
  ScrollProgress.prototype._getProgress = function() {

      let x = typeof window.scrollX === 'undefined'
          ? window.pageXOffset
          : window.scrollX;

      let y = typeof window.scrollY === 'undefined'
          ? window.pageYOffset
          : window.scrollY;

      /** Height of specific part. */
      if ( t42ReadingProgressOptions.selector && document.querySelector( t42ReadingProgressOptions.selector ) !== null ) {

          /** Progress for selector */
          let scrolled = window.pageYOffset || (document.documentElement || document.body.parentNode || document.body).scrollTop;
          let windowHeight = window.innerHeight || document.documentElement.clientHeight;

          const selector = document.querySelector( t42ReadingProgressOptions.selector );

          let selectorOffsetY = selector.getBoundingClientRect().top - document.body.getBoundingClientRect().top;
          let selectorStart = scrolled - ( selectorOffsetY - windowHeight );
          let selectorEnd = scrolled - ( selectorOffsetY + selector.getBoundingClientRect().height - windowHeight );

          /** Selector in the first screen. */
          if ( selectorOffsetY < windowHeight ) {

              selectorStart = scrolled;

          }

          try {

              if ( selectorStart >= 0 && selectorEnd <= 0 ) {

                  y = selectorStart / selector.getBoundingClientRect().height;

              } else {

                  if ( selectorStart < 0 ) {

                      y = 0;

                  } else if ( selectorEnd > 100 ) {

                      y = 100;

                  }

              }

              return {
                  x: this._viewportWidth === 0 ? 0 : x / this._viewportWidth,
                  y: y
              };

          } catch ( e ) {

              console.error( e );

          }

      }

      /** Whole page height. */
      else {

          return {
              x: this._viewportWidth === 0 ? 0 : x / this._viewportWidth,
              y: this._viewportHeight === 0 ? 0 : y / this._viewportHeight
          };

      }


  };

  /**
   * Get scroll progress on both axis
   * @method _getProgress
   * @returns {undefined}
   */
  ScrollProgress.prototype._onScroll = function() {
    this._progress = this._getProgress();
    this._handleUpdate(this._progress.x, this._progress.y);
  };

  /**
   * Update viewport metrics, recalculate progress and call update callback
   * @method _onResize
   * @returns {undefined}
   */
  ScrollProgress.prototype._onResize = function() {
    this._viewportHeight = this._getViewportHeight();
    this._viewportWidth = this._getViewportWidth();

    this._progress = this._getProgress();

    // trigger update function
    this._handleUpdate(this._progress.x, this._progress.y);
  };

  /**
   * Trigger update callback
   * @method trigger
   * @returns {undefined}
   */
  ScrollProgress.prototype.trigger = function() {
    this._handleUpdate(this._progress.x, this._progress.y);
  };

  /**
   * Destroy scroll observer, remove listeners and update callback
   * @method destroy
   * @returns {undefined}
   */
  ScrollProgress.prototype.destroy = function() {
    window.removeEventListener('scroll', this._onScroll);
    window.removeEventListener('resize', this._onResize);
    this._handleUpdate = null;
  };

  return ScrollProgress;

})));

/**
 * Fires when the initial HTML document has been completely loaded and parsed.
 **/
document.addEventListener("DOMContentLoaded", function() {

    /**
     * Create DOM elements which graphically represent the progress.
     *
     * @typedef {Object} t42ReadingProgressOptions
     **/
    function _createElements() {

        /** Progress Bar wrapper element. */
        let progressWrapper = document.createElement( 'div' );
        progressWrapper.id = 't42-reading-progress-wrapper';
        progressWrapper.className = 't42-style-' + t42ReadingProgressOptions.style;

        /** Progress Bar element. */
        let progressElement = document.createElement( 'div' );
        progressElement.id = 't42-reading-progress';
        progressElement.style.height = t42ReadingProgressOptions.height + 'px';
        progressElement.style.position = 'fixed';
        if ( 'top' === t42ReadingProgressOptions.position ) { progressElement.style.top = t42ReadingProgressOptions.offset + 'px'; }
        if ( 'bottom' === t42ReadingProgressOptions.position ) { progressElement.style.bottom = t42ReadingProgressOptions.offset + 'px'; }
        progressElement.style.background = t42ReadingProgressOptions.color;

        /** Add Progress Bar to document body. */
        progressWrapper.appendChild( progressElement );
        document.body.appendChild( progressWrapper );

    }
    _createElements();

    /** Run Progress Bar. */
    const progressElement = document.querySelector( '#t42-reading-progress' );

    const progressObserver = new ScrollProgress( ( x, y ) => {
        progressElement.style.width = y * 100 + '%';
    } );

} );

