<?php
/**
 * Reading Progress. Exclusively on Envato Market: https://1.envato.market/42themeCC
 * @encoding     UTF-8
 * @version      2.0.0
 * @copyright    Copyright (C) 2016 - 2021 42Theme (https://42theme.com). All rights reserved.
 * @license      Envato Standard Licenses
 * @author       Alexander Khmelnitskiy
 * @support      support@42theme.com
 **/

namespace T42\ReadingProgress;

/** Exit if accessed directly. */
if ( ! defined( 'ABSPATH' ) ) {
	header( 'Status: 403 Forbidden' );
	header( 'HTTP/1.1 403 Forbidden' );
	exit;
}

/**
 * SINGLETON: Settings class used to modify default plugin settings.
 *
 * @since 2.0.0
 * @author Alexander Khmelnitskiy
 **/
final class Config {

    use Singleton;

	/**
	 * Prepare list of critical initial checks to run.
	 * Available checks: 'php56', 'curl'
	 *
	 * @since 2.0.0
	 * @access public
	 *
	 * @return array
	 **/
	public function get_initial_checks() {

		return [
			'php56', // Plugin need PHP 5.6+.
			'curl' // Plugin need cURL extension.
 		];

	}

	/**
	 * Prepare plugin features configurations by modifying the defaults.
	 *
	 * @since 2.0.0
	 * @access public
	 *
	 * @return void
	 **/
	private function prepare_config() {

		/** Get default plugin config. */
		$config = Plugin::get_config();

		/** Merlin WP. */
		$config['merlin']['enabled'] = true;

		/** Update config. */
		Plugin::set_config( $config );

	}

    /**
     * Prepare plugin settings by modifying the default one.
     *
     * @since 2.0.0
     * @access public
     *
     * @return void
     **/
    public function prepare_settings() {

	    /** Prepare custom plugin features configurations. */
	    $this->prepare_config();

        /** Get default plugin tabs. */
        $tabs = Plugin::get_tabs();

        /** Remove 'Delete plugin, settings and data' option from Uninstall tab. */
        unset( $tabs['uninstall']['fields']['delete_plugin']['options']['plugin+settings+data'] );

	    /** Create General tab. */
	    $tabs = $this->create_tab_general( $tabs );

        /** Set updated tabs. */
        Plugin::set_tabs( $tabs );

        /** Refresh settings. */
        Settings::get_instance()->refresh_options();

    }

	/**
	 * Create General tab.
	 *
	 * @param array $tabs - List of tabs with all settings and fields.
	 *
	 * @since 2.0.0
	 * @access private
	 *
	 * @return array - List of tabs with all settings and fields.
	 **/
	private function create_tab_general( $tabs ) {

        /** Add Reset Settings to tab. */
        $tabs['general']['reset_settings'] = true;

        /** Short hand. */
        $fields = &$tabs[ 'general' ][ 'fields' ];

        /** Preview. */
        $fields[ 'preview' ] = [
            'type'             => 'preview',
            'render'           => [ Preview::get_instance(), 'render_preview' ],
            'label'            => esc_html__( 'Preview:', 't42-reading-progress' ),
            'show_label'       => true,
            'description'      => esc_html__( 'Change settings to see how the progress bar will look.', 't42-reading-progress' ),
            'show_description' => true,
            'default'          => '',
        ];

        /** Divider. */
        $fields['preview_divider'] = [ 'type' => 'divider' ];

        /** Style. */
        $fields['style'] = [
            'type'              => 'select',
            'label'             => esc_html__( 'Style', 't42-reading-progress' ),
            'show_label'        => true,
            'placeholder'       => esc_html__( 'Select Style', 't42-reading-progress' ),
            'description'       => esc_html__( 'Choose the style for Reading Progress Bar.', 't42-reading-progress' ),
            'show_description'  => true,
            'default'           => ['default'],
            'options'           => [
                'default'  => esc_html__( 'Default', 't42-reading-progress' ),
                'rounded'  => esc_html__( 'Rounded', 't42-reading-progress' ),
                'plastic'  => esc_html__( 'Plastic', 't42-reading-progress' ),
                'animated' => esc_html__( 'Animated', 't42-reading-progress' ),
                'shine'    => esc_html__( 'Shine', 't42-reading-progress' ),
                'glow'     => esc_html__( 'Glow', 't42-reading-progress' ),
            ],
        ];

        /** Color. */
        $fields['color'] = [
            'type'              => 'colorpicker',
            'label'             => esc_html__( 'Color', 't42-reading-progress' ),
            'show_label'        => true,
            'description'       => esc_html__( 'Select color or gradient for Progress Bar.', 't42-reading-progress' ),
            'show_description'  => true,
            'default'           => '#0577e8',
            'gradient_mode'     => true,
            'gradient_switcher' => 'on',
            'gradient_default'  => 'linear-gradient(0deg, rgba(119, 128, 227, 1) 0%,rgba(32, 182, 221, 1) 100%)',
            'gradient_presets'  => [
                'linear-gradient(0deg, rgba(119, 128, 227, 1) 0%,rgba(32, 182, 221, 1) 100%)',
                'linear-gradient(to right, #c92c2c 0%,#eae42c 50%,#3cad26 100%)',
                'linear-gradient(to right, #4cd964, #5ac8fa, #007aff, #34aadc, #5856d6, #ff2d55)',
                'linear-gradient(135deg, #fad961 0%, #f76b1c 100%)',
                'linear-gradient(135deg, #7117ea 0%, #ea6060 100%)',
                'linear-gradient(135deg, #F5515F 0%, #A1051D 100%)',
                'linear-gradient(135deg, #0FF0B3 0%, #036ED9 100%)',
                'linear-gradient(135deg, #f65599 0%, #4d0316 100%)',
                'linear-gradient(135deg, #FDEB71 10%, #F8D800 100%)',
                'linear-gradient(135deg, #A0FE65 10%, #FA016D 100%)',
                'linear-gradient(135deg, #F6CEEC 10%, #D939CD 100%)',
                'linear-gradient(135deg, #FDD819 10%, #E80505 100%)',
                'linear-gradient(135deg, #EECE13 10%, #B210FF 100%)',
                'linear-gradient(135deg, #81FFEF 10%, #F067B4 100%)',
            ],
        ];

        /** Shadow. */
        $fields['shadow'] = [
            'type'              => 'box_shadow',
            'label'             => esc_html__( 'Shadow', 't42-reading-progress' ),
            'show_label'        => true,
            'description'       => esc_html__( 'Customize shadow for progress bar.', 't42-reading-progress' ),
            'show_description'  => true,
            'default'           => [
                'active'     => '0',
                'color'      => 'rgba(0, 0, 0, 0.5)',
                'horizontal' => 0,
                'vertical'   => 0,
                'blur'       => 10,
                'spread'     => 0,
                'position'   => ' ', //inset
            ],
            'attr'              => [
                'class'     => 'test-class',
            ]
        ];

        /** Height. */
        $fields['height'] = [
            'type'              => 'range_slider',
            'label'             => esc_html__( 'Height', 't42-reading-progress' ),
            'show_label'        => true,
            'description'       => esc_html__( 'Select height in pixels for Progress Bar.', 't42-reading-progress' ),
            'show_description'  => true,
            'min'               => 1,
            'max'               => 100,
            'step'              => 1,
            'default'           => 10,
            'postfix'           => ' px',
        ];

        /** Position buttons. */
        $fields['position_buttons'] = [
            'type'              => 'button_group',
            'label'             => esc_html__( 'Position', 't42-reading-progress' ),
            'show_label'        => true,
            'description'       => esc_html__( 'Select position where to show Progress Bar.', 't42-reading-progress' ),
            'show_description'  => true,
            'group'             => [
                [
                    'button_text'       => esc_html__( 'Top of page', 't42-reading-progress' ),
                    'icon'              => 'arrow-up',
                    'attr'              => [
                        'class'         => 't42-position-btn t42-button-primary t42-waves-effect t42-waves-dark',
                        'data-position' => 'top',
                    ]
                ],
                [
                    'button_text'       => esc_html__( 'At the bottom', 't42-reading-progress' ),
                    'icon'              => 'arrow-down',
                    'attr'              => [
                        'class'         => 't42-position-btn t42-button-default t42-waves-effect t42-waves-dark',
                        'data-position' => 'bottom',
                    ]
                ],
            ]
        ];

        /** Position hidden input. */
        $fields['position'] = [
            'type'    => 'hidden',
            'default' => 'top',
        ];

        /** Offset. */
        $fields['offset'] = [
            'type'              => 'range_slider',
            'label'             => esc_html__( 'Offset', 't42-reading-progress' ),
            'show_label'        => true,
            'description'       => esc_html__( 'Distance to move progress bar up or down, depending on the selected position.', 't42-reading-progress' ),
            'show_description'  => true,
            'min'               => 0,
            'max'               => 500,
            'step'              => 1,
            'default'           => 0,
            'postfix'           => ' px',
        ];

        /** Divider. */
        $fields['offset_divider'] = [ 'type' => 'divider' ];

        /** Selector. */
        $fields['selector'] = [
			'type'              => 'text',
			'label'             => esc_html__( 'Custom Selector', 't42-reading-progress' ),
			'show_label'        => true,
			'reset_settings'    => false,
			'placeholder'       => esc_html__( '.wp-post > article', 't42-reading-progress' ),
			'description'       => esc_html__( 'Enter valid CSS selector to display the progress bar not for the entire page height, but for a specific part. Leave blank to use full page height.', 't42-reading-progress' ),
			'show_description'  => true,
			'default'           => '',
			'attr'              => [
				'maxlength' => '4500',
				'prefix'    => esc_html__( 'CSS Selector:', 't42-reading-progress' ),
			]
		];

		return $tabs;

	}

} // End class Config.
